/******************************************************************************
 * @file     main.c
 * @brief    Main routine of buring Boot code to LDROM, and IAP code to APROM.
 * @version  1.0.0
 * @date     18, October, 2012
 *
 * @note
 * Copyright (C) 2012-2014 Nuvoton Technology Corp. All rights reserved.
 ******************************************************************************/
#include <stdio.h>
#include "NUC1xx.h"
#include "Driver\DrvUART.h"
#include "Driver\DrvSYS.h"
#include "Driver\DrvFMC.h"


#define MyLDROM_BASE  	0x00100000
#define MyLDROM_SIZE		0x1000
#define ISP_BASE		(50*1024)
//#define ISP_BASE		(0x4000)
#define ISP_MAX_SIZE	(14*1024)
#define PAGE_SIZE   	512

#define CONFIG0 0x300000
#define CONFIG1 0x300004

extern uint32_t loaderImage1Base, loaderImage1Limit; 
extern uint32_t loaderImage2Base, loaderImage2Limit;

static int  load_image_to_flash(uint32_t image_base, uint32_t image_limit, uint32_t flash_addr, uint32_t max_size);


/*---------------------------------------------------------------------------------------------------------*/
/*  Main Function									                                           			   */
/*---------------------------------------------------------------------------------------------------------*/
int32_t main (void)
{
	STR_UART_T sParam;
	uint32_t	config0, config1;
	int			cbs;
	volatile int  loop;
	
	/* Set UART Pin */
	outpw(&SYS->GPBMFP, inpw(&SYS->GPBMFP) | (0x3<<(0)));
	
	/* UART Setting */
    sParam.u32BaudRate 		= 115200;
    sParam.u8cDataBits 		= DRVUART_DATABITS_8;
    sParam.u8cStopBits 		= DRVUART_STOPBITS_1;
    sParam.u8cParity 		= DRVUART_PARITY_NONE;
    sParam.u8cRxTriggerLevel= DRVUART_FIFO_1BYTES;

	/* Set UART Configuration */
 	if(DrvUART_Open(UART_PORT0,&sParam) != E_SUCCESS)
	{
		printf("UART0 open failed\n");
		return FALSE;
	}

    printf("\n\n");
    printf("+--------------------------------------------------+\n");
    printf("|         User program running on APROM            |\n");
    printf("+--------------------------------------------------+\n");

	UNLOCKREG();

	/* Enable ISP function */
    FMC->ISPCON.ISPEN = 1;

	printf("Loading ld_boot image to LDROM...\n");
	DrvFMC_EnableLDUpdate();
	if (load_image_to_flash((uint32_t)&loaderImage1Base, (uint32_t)&loaderImage1Limit, 
					MyLDROM_BASE, MyLDROM_SIZE) != 0)
	{
		printf("Load image to LDROM failed!\n");
		return -1;
	}
	DrvFMC_DisableLDUpdate();
	
	printf("Loading ap_loader image to APROM 50KB offset...\n");
	DrvFMC_EnableAPUpdate();
	if (load_image_to_flash((uint32_t)&loaderImage2Base, (uint32_t)&loaderImage2Limit, 
					ISP_BASE, ISP_MAX_SIZE) != 0)
	{
		printf("Load image to APROM failed!\n");
		return -1;
	}
	DrvFMC_DisableAPUpdate();
	
	/*
	 *  Modify CBS
	 */
    if ((DrvFMC_Read(CONFIG0, &config0) < 0) || (DrvFMC_Read(CONFIG1, &config1) < 0))
    {
    	printf("\n\nFailed to read Config!\n\n");
    	return -1; 
    }
    cbs = (config0 >> 6) & 0x3;
    printf("Config0 = 0x%x, Config1 = 0x%x, CBS=%d\n\n", config0, config1, cbs); 

	if (cbs != 0)
	{
		DrvFMC_EnableConfigUpdate();
		config0 = (config0 & ~0xc0);		/* set CBS to 00b */
		DrvFMC_Erase(CONFIG0);
		DrvFMC_Write(CONFIG0, config0 | 0x2);
		DrvFMC_Write(CONFIG1, config1);
		DrvFMC_DisableConfigUpdate();
	}
	printf("\nExecute chip reset, will boot from LDROM with VECMAP...\n\n");
    for (loop = 0; loop < 0x10000; loop++) ;
	
	// do chip reset
	DrvSYS_ResetChip();

	return 0;
}


static int  load_image_to_flash(uint32_t image_base, uint32_t image_limit, uint32_t flash_addr, uint32_t max_size)
{
	uint32_t   i, j, u32Data, u32ImageSize, *pu32Loader;
	
    u32ImageSize = image_limit - image_base;
    if (u32ImageSize == 0)
    {
        printf("  ERROR: Loader Image is 0 bytes!\n");
        return -1;
    }

    if (u32ImageSize > max_size)
    {
        printf("  ERROR: Loader Image is larger than %d KBytes!\n", max_size/1024);
        return -1;    
    }

    printf("Program image to flash address 0x%x...", flash_addr);
    pu32Loader = (uint32_t *)image_base;
    for (i = 0; i < u32ImageSize; i += PAGE_SIZE)
    {
        if (DrvFMC_Erase(flash_addr + i))
        {
        	printf("Erase failed on 0x%x\n", flash_addr + i);
        	return -1;
        }
        
        for (j = 0; j < PAGE_SIZE; j += 4)
        {
            if (DrvFMC_Write(flash_addr + i + j, pu32Loader[(i + j) / 4]))
            {
            	printf("Write faild on 0x%x\n", flash_addr + i + j);
            	return -1; 
            }
        }
    }
    printf("OK.\n");
    
    printf("Verify ...");

    /* Verify loader */
    for (i = 0; i < u32ImageSize; i += PAGE_SIZE)
    {
        for (j = 0; j < PAGE_SIZE; j += 4)
        {
            if (DrvFMC_Read(flash_addr + i + j, &u32Data))
            {
            	printf("Read failed on 0x%x\n", flash_addr + i + j);
            	return -1;
            }
            if (u32Data != pu32Loader[(i+j)/4])
            {
            	printf("data mismatch on 0x%x, [0x%x], [0x%x]\n", flash_addr + i + j, u32Data, pu32Loader[(i+j)/4]);
            	return -1;  
            }
            
            if (i + j >= u32ImageSize)
                break;
        }
    }
    printf("OK.\n");
    return 0;
}

/*** (C) COPYRIGHT 2012 Nuvoton Technology Corp. ***/
